﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using System;

public class GameController : MonoBehaviour {

	private float levelTime;
	private bool timeStarted;
	private bool timeStopped;
    private int hardResetCounter;
    private int activeCheckpointNumber;
    private GameObject activeCheckpoint;
    private CameraManager cameraManager;
    public List<GameObject> resettableObjects;
    public GameObject startPoint;
    public GameObject playerObject;
    private Rigidbody2D playerRigidBody;
    private PlayerController playerController;
    public Text timerText;
	// Use this for initialization
	void Start () {
        hardResetCounter = 0;
        levelTime = 0;
        activeCheckpointNumber = 0;
		timeStarted = false;
		timeStopped = false;
        playerRigidBody = playerObject.GetComponent<Rigidbody2D>();
        if(playerRigidBody == null)
        {
            Debug.Log("Could not find Rigid Body of the player");
        }
        playerController = playerObject.GetComponent<PlayerController>();
        if (playerController == null)
        {
            Debug.Log("Could not find playerController of the player");
        }

        cameraManager = GetComponent<CameraManager>();
        if(cameraManager == null)
        {
            Debug.Log("Could not find camera manager");
        }
        resetGame(true);
    }

    public void resetGame(bool hardReset)
    {
        if(activeCheckpointNumber == 0 || hardReset)
        {
            timeStarted = true;
            timeStopped = false;
            activeCheckpointNumber = 0;
            foreach (GameObject ro in resettableObjects)
            {
                IResettableObject resettable = (IResettableObject)ro.GetComponent(typeof(IResettableObject));
                if (resettable != null)
                {
                    resettable.resetObject();
                }
            }
            playerObject.transform.position = startPoint.transform.position;
            playerRigidBody.velocity = new Vector2(0, 0);
            playerRigidBody.angularVelocity = 0;
            levelTime = 0;
        }
        else
        {
            playerObject.transform.position = activeCheckpoint.transform.position;
            foreach(GameObject ro in resettableObjects)
            {
                IResettableObject resettable = (IResettableObject)ro.GetComponent(typeof(IResettableObject));
               if(resettable != null)
                { 
                    resettable.resetObject();
                }
            }
        }
        playerRigidBody.velocity = new Vector2(0,0);
        playerRigidBody.angularVelocity = 0;  

    }

    public bool trySetNewActiveCheckpoint(GameObject checkpoint, int checkpointNumber)
    {
        if (checkpointNumber > activeCheckpointNumber)
        {
            activeCheckpoint = checkpoint;
            activeCheckpointNumber = checkpointNumber;
            return true;
        }

        return false;
    }

    public void stopTimer()
	{
        timeStopped = true;
	}

	public void startTimer()
	{
		timeStarted = true;
	}
    public void incrementHardResetCounter()
    {
        hardResetCounter++;
        if (hardResetCounter > 20)
        {
            resetGame(true);
            hardResetCounter = 0;
        }
    }

	void FixedUpdate () 
	{
        if (Input.GetKeyDown(KeyCode.R))
        {
            resetGame(true);
        }

        if(Math.Abs(playerRigidBody.velocity.x) > playerController.maxManualSpeed + 2)
        {
            cameraManager.zoomOut();
        }
        else
        {
            cameraManager.zoomIn();
        }

        cameraManager.offsetPercent = Math.Abs(playerRigidBody.velocity.x / playerController.maxSpeedX) * (playerRigidBody.velocity.x >=0? 1:-1);

        if (timeStarted && !timeStopped) 
		{
			levelTime += Time.fixedDeltaTime;
            int minutes = (int)(levelTime / 60f) % 60;
            int seconds = (int)(levelTime % 60f);
            int milliseconds = (int)(levelTime * 1000f) % 1000 / 10;
            String millisecondsString = milliseconds.ToString("D2");
            timerText.text = minutes.ToString("D1") + ":" + seconds.ToString("D2") + ":" +millisecondsString;
		}
	}
}
