﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class PlayerController : MonoBehaviour {


    private Vector2 jumpTimer;
    private Vector2 spinTimer;
    // Use this for initialization
    public float maxManualSpin;
    public float maxManualSpeed;
    public float maxSpinSpeed;
    public float maxSpeedX;
    public float maxSpeedY;
    public float torque;
    public float acceleration;
    public float jumpPower;
    public float jumpCooldown;
    public float spinCooldown;
    public float spinPower;
	public GameController gameController;
    public FixRotationPod fixRotationPod;
    public Text debugTiltText;
    private Rigidbody2D rb;
    private Collider2D collider2D;
    private bool isGrounded;
    private bool isJumpButtonPressed;
    private int  spinPressedDirection;

    void Start()
    {
        collider2D = GetComponent<Collider2D>();
        rb = GetComponent<Rigidbody2D>();
        isGrounded = false;
        jumpTimer.x = 0;
        jumpTimer.y = jumpCooldown;
        spinTimer.x = 0;
        spinTimer.y = spinCooldown;
		if (gameController) {
			gameController.startTimer();
		} else 
		{
			Debug.Log ("No Gamecontroller assigned to Player Object!");
		}
    }

    void FixedUpdate()
    {
        float turn = Input.GetAxis("Horizontal");
        if (turn == 0)
        {
            turn = Input.acceleration.x * 3;
        }

        turn = Mathf.Clamp(turn, -1, 1);
        fixRotationPod.tiltPercent = turn;
        debugTiltText.text = "Tilt XAxis: " + turn;

        if ((-rb.angularVelocity < maxManualSpin && turn > 0) ||
           (-rb.angularVelocity > -maxManualSpin && turn < 0))
        {
            rb.AddTorque(torque * -turn);
        }

        if((rb.velocity.x < maxManualSpeed && turn > 0)||
           (rb.velocity.x > -maxManualSpeed && turn < 0))
        { 
            rb.AddForce(new Vector2(acceleration * turn, 0));
        }

        if (Input.GetKeyDown(KeyCode.Space) || Input.GetKey(KeyCode.Space)
            || isJumpButtonPressed)
        {
            tryJump();
        }

        if (Input.GetKeyDown(KeyCode.S) || spinPressedDirection == 1)
        {
            trySpin(1);
        }

        if (Input.GetKeyDown(KeyCode.W) || spinPressedDirection == -1)
        {
            trySpin(-1);
        }

        processTimers();
        restrictValues();
    }

    void OnCollisionEnter2D(Collision2D collision)
    {
        isGrounded = hitBottom(collision);
    }

    void OnCollisionStay2D(Collision2D collision)
    {
        isGrounded = hitBottom(collision);
    }

    private bool hitBottom(Collision2D collision)
    {
        Collider2D collider = collision.collider;

        Vector3 contactPoint = collision.contacts[0].point;
        Vector3 center = GetComponent<CircleCollider2D>().bounds.center;
        return contactPoint.y < center.y - GetComponent<CircleCollider2D>().bounds.size.y/4;
    }

    private void OnCollisionExit2D(Collision2D collision)
    {
        isGrounded = false;
    }

    public void tryJump()
    {
        if(jumpTimer.x <= 0 && isGrounded)
        {
            jumpTimer.x = jumpTimer.y;
            jump();
        }
    }

    private void jump()
    {
        rb.AddForce(new Vector2(0, jumpPower));
    }

    public void trySpin(int direction)
    {
        if (spinTimer.x <= 0 && !isGrounded)
        {
            spinTimer.x = spinTimer.y;
            spin(direction);
        }
    }

    private void spin(int direction)
    {
        rb.angularVelocity = 0;
        rb.AddTorque(spinPower * direction);
    }

    private void processTimers()
    {
        float deltaTime = Time.fixedDeltaTime;
        if(jumpTimer.x > 0)
        {
            jumpTimer.x -= deltaTime;
            Mathf.Clamp(jumpTimer.x, 0, jumpCooldown);
        }
        if (spinTimer.x > 0)
        {
            spinTimer.x -= deltaTime;
            Mathf.Clamp(spinTimer.x, 0, spinCooldown);
        }
    }

    private void restrictValues()
    {
        rb.velocity = new Vector2(Mathf.Clamp(rb.velocity.x, -maxSpeedX, maxSpeedX), Mathf.Clamp(rb.velocity.y, -100, maxSpeedY));
        Mathf.Clamp(rb.angularVelocity, -maxSpinSpeed, maxSpinSpeed);
    }

    // Android Controls start here
    public void androidTryJump(bool isJumpPressed)
    {
        isJumpButtonPressed = isJumpPressed;
    }

    // Android Controls start here
    public void androidTrySpin(int direction)
    {
        spinPressedDirection = direction;
    }
}
